<?php

namespace FiloBlu\ExtBlackbirdMenuManager\Setup;

use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\Setup\ModuleContextInterface;

/**
 * @codeCoverageIgnore
 */
class UpgradeSchema implements UpgradeSchemaInterface {

    /**
     * {@inheritdoc}
     */
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context) {

        if (!$context->getVersion()) {
            return $this;
        }

        if (version_compare($context->getVersion(), '1.1.0') < 0) {

            $installer = $setup;

            $installer->startSetup();

            $connection = $installer->getConnection();
            
            $table = $installer->getTable('blackbird_menumanager_node');
            
            if ($connection->tableColumnExists($table, "customer_group") === false) {
                $columns = [
                    'customer_group' => [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => 255,
                        'nullable' => true,
                        'comment' => 'Customer Group'
                    ],
                ];
                
                foreach ($columns as $name => $definition) {
                    $connection->addColumn($table, $name, $definition);
                }
            }
            
            $table = $installer->getTable('blackbird_menumanager_menu');
            
            if ($connection->tableColumnExists($table, "customer_group") === false) {
                $columns = [
                    'customer_group' => [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => 255,
                        'nullable' => true,
                        'comment' => 'Customer Group'
                    ],
                ];
                
                foreach ($columns as $name => $definition) {
                    $connection->addColumn($table, $name, $definition);
                }
            }

            $installer->endSetup();
        }

        if (version_compare($context->getVersion(), '1.2.0') < 0) {

            $installer = $setup;

            $installer->startSetup();

            $connection = $installer->getConnection();
            
            $table = $installer->getTable('blackbird_menumanager_node');
            
            if ($connection->tableColumnExists($table, "max_item_column") === false) {

                $columns = [
                    'max_item_column' => [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'comment' => 'Max items for column'
                    ],
                ];


                foreach ($columns as $name => $definition) {
                    $connection->addColumn($table, $name, $definition);
                }
            }

            $installer->endSetup();
        }
    }

}
